// Box.h: interface for the Box class.
//
//////////////////////////////////////////////////////////////////////

#ifndef __BOX_H
#define __BOX_H

#include "Real.h"
#include "Point3D.h"
#include <assert.h>

real max_real(real x, real y);

/** This class represents a 3D box aligned with the axes.
* @doc
* left < right on the X axis
* back < front on the Y axis
* bottom < top on the Z axis
*/
class Box  
{
public:
	///
	Box() throw();
	///
	Box(real left, real right, 
		real back, real front, 
		real bottom, real top) throw();
	///
	Box(const Box & rhs) throw();
	///
	~Box() throw();

	///
	void operator=(const Box & rhs) throw();

	///
	real GetLeft() const throw();
	///
	real GetRight() const throw();
	///
	real GetFront() const throw();
	///
	real GetBack() const throw();
	///
	real GetBottom() const throw();
	///
	real GetTop() const throw();
	///
	real GetBaseArea() const throw();
	///
	real GetMaxEdgeLength() const throw();
	///
	Point3D GetCenter() const throw();


private:
	real m_left;
	real m_right;
	real m_back;
	real m_front;
	real m_bottom;
	real m_top;
};


inline real max_real(real x, real y) {
	return (x > y) ? x : y;
}


// Inline Implementations

/** @author Roman Manevich
* @version 7.7.2000
*/
inline Box::Box() throw()
{
}

/** @author Roman Manevich
* @version 7.7.2000
*/
inline Box::Box(const Box & rhs) throw()
{
	operator=(rhs);
}

/** @author Roman Manevich
* @version 7.7.2000
*/
inline Box::Box(real left, real right, 
	real back, real front,
	real bottom, real top) throw() :
m_left(left),
m_right(right),
m_back(back),
m_front(front),
m_bottom(bottom),
m_top(top)
{
	assert(left <= right &&
		back <= front &&
		bottom <= top);
}

/** @author Roman Manevich
* @version 7.7.2000
*/
inline Box::~Box() throw()
{
}

/** @author Roman Manevich
* @version 7.7.2000
*/
inline void Box::operator=(const Box & rhs) throw()
{
	m_left = rhs.m_left;
	m_right = rhs.m_right;
	m_back = rhs.m_back;
	m_front = rhs.m_front;
	m_bottom = rhs.m_bottom;
	m_top = rhs.m_top;
}

/** @author Roman Manevich
* @version 7.7.2000
*/
inline real Box::GetLeft() const throw()
{
	return m_left;
}

/** @author Roman Manevich
* @version 7.7.2000
*/
inline real Box::GetRight() const throw()
{
	return m_right;
}

/** @author Roman Manevich
* @version 7.7.2000
*/
inline real Box::GetFront() const throw()
{
	return m_front;
}

/** @author Roman Manevich
* @version 7.7.2000
*/
inline real Box::GetBack() const throw()
{
	return m_back;
}

/** @author Roman Manevich
* @version 7.7.2000
*/
inline real Box::GetBottom() const throw()
{
	return m_bottom;
}

/** @author Roman Manevich
* @version 7.7.2000
*/
inline real Box::GetTop() const throw()
{
	return m_top;
}

inline real Box::GetBaseArea() const throw()
{
	return (m_right - m_left)*(m_front - m_back);
}

inline real Box::GetMaxEdgeLength() const throw()
{
	return max_real( max_real(m_right - m_left, m_front - m_back), m_top - m_bottom);
}

inline Point3D Box::GetCenter() const throw()
{
	return Point3D(0.5*(m_right + m_left), 0.5*(m_front + m_back), 0.5*(m_top + m_bottom));
}

#endif // __BOX_H