#ifndef __TRIANGLE_H
#define __TRIANGLE_H

#include <assert.h>
#include "Point3D.h"
#include "Defs.h"
// #include "Segment.h"

/** This class represents a triangle in 3D.
*/
class Triangle
{
public:
	///
	Triangle() throw() {};
	/// Copy Constructor
	Triangle(const Triangle & rhs) throw();
	/// Assignment operator
	void operator=(const Triangle & rhs) throw();

	bool operator!=(const Triangle & rhs) throw();
	bool operator==(const Triangle & rhs) throw();
	///
	Triangle(const Point3D & v1,const Point3D & v2,const Point3D & v3) throw();

	~Triangle() throw() {};

	///
	const Point3D & GetFirstVertex() const throw();
	///
	const Point3D & GetSecondVertex() const throw();
	///
	const Point3D & GetThirdVertex() const throw();
	///
	const Point3D & GetVertex(USHORT ind) const throw();

	///
	void SetFirstVertex(const Point3D & newVertex) throw();
	///
	void SetSecondVertex(const Point3D & newVertex) throw();
	///
	void SetThirdVertex(const Point3D & newVertex) throw();
	///
	real GetArea() const throw();

/*	
	///
	static bool DoTrianglesIntersect(const Triangle & triangle1,
		const Triangle & triangle2);
*/

/*
	/// The assumption is that the triangle lies completly on xy plane 
	static bool DoesTriangleIntersectSegment(const Triangle & triangle,
		const Segment & segment);
*/
	/// by Olga
	void Translate(const Point3D & translateVector);
	///
	Triangle GetTranslated(const Point3D & translateVector) const throw();

	/// by Olga - only in xy plane
	void RotateClockwiseAboutVertex(USHORT vertexInd, real angle);
	/// by Olga - only in xy plane
	void RotateCounterClockwiseAboutVertex(USHORT vertexInd, real angle);

	
	/// by Olga - this function returns true if the given
	/// triangle is fully contained inside *this. 
	/// Pre-condition: both triangle and *this are on xy plane
	bool ContainsTriangle(const Triangle & triangle);

	/// by Olga - this function returns true if the given
	/// point is inside *this
	/// Pre-condition: both point and *this are on xy plane
	bool ContainsPoint(const Point3D & point);

private:
	///
	Point3D m_vertexPoints[3];
};


//////////////////////////////////////////////////////////////////////
// Inline Implementations.
//////////////////////////////////////////////////////////////////////

/** @author Roman Manevich
* @version 6.7.2000
*/
inline Triangle::Triangle(const Point3D & v1,const Point3D & v2,const Point3D & v3) throw()
{
	m_vertexPoints[0] = v1;
	m_vertexPoints[1] = v2;
	m_vertexPoints[2] = v3;

	// Check that the three points are not co-linear
	// assert( GetArea() > EPSILON );
}

inline const Point3D & Triangle::GetFirstVertex() const throw()
{
	return m_vertexPoints[0];
}

inline const Point3D & Triangle::GetSecondVertex() const throw()
{
	return m_vertexPoints[1];
}

inline const Point3D & Triangle::GetThirdVertex() const throw()
{
	return m_vertexPoints[2];
}

inline void Triangle::SetFirstVertex(const Point3D & newVertex) throw()
{
	m_vertexPoints[0] = newVertex;
}

inline void Triangle::SetSecondVertex(const Point3D & newVertex) throw()
{
	m_vertexPoints[1] = newVertex;
}

inline void Triangle::SetThirdVertex(const Point3D & newVertex) throw()
{
	m_vertexPoints[2] = newVertex;
}

/** @author Roman Manevich
* @version 6.7.2000
*/
inline Triangle::Triangle(const Triangle & rhs) throw()
{
	m_vertexPoints[0] = rhs.m_vertexPoints[0];
	m_vertexPoints[1] = rhs.m_vertexPoints[1];
	m_vertexPoints[2] = rhs.m_vertexPoints[2];
}

/** @author Roman Manevich
* @version 6.7.2000
*/
inline void Triangle::operator=(const Triangle & rhs) throw()
{
	m_vertexPoints[0] = rhs.m_vertexPoints[0];
	m_vertexPoints[1] = rhs.m_vertexPoints[1];
	m_vertexPoints[2] = rhs.m_vertexPoints[2];
}

inline bool Triangle::operator==(const Triangle & rhs) throw()
{
	return (m_vertexPoints[0] == rhs.m_vertexPoints[0]) &&
		   (m_vertexPoints[1] == rhs.m_vertexPoints[1]) &&
		   (m_vertexPoints[2] == rhs.m_vertexPoints[2]);
}

inline bool Triangle::operator!=(const Triangle & rhs) throw()
{
	return !(*this == rhs);
}

/** @author Roman Manevich
* @version 6.7.2000
*/
inline real Triangle::GetArea() const throw()
{
	Vector3D normal = Point3D::VectorProduct(GetSecondVertex() - GetFirstVertex(),
		GetThirdVertex() - GetFirstVertex());
	return normal.Size() /2;//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!11
	//return normal.SquareOfSize();
}

/// by Olga
inline void Triangle::Translate(const Point3D & translateVector)
{
	m_vertexPoints[0] += translateVector;
	m_vertexPoints[1] += translateVector;
	m_vertexPoints[2] += translateVector;
}

/// by Olga - this function returns true if the given
/// point is inside *this
/// Pre-condition: both point and *this are on xy plane
inline bool Triangle::ContainsPoint(const Point3D & point)
{
	// Triangles' edges
	Point3D AB = m_vertexPoints[1] - m_vertexPoints[0];
	Point3D BC = m_vertexPoints[2] - m_vertexPoints[1];
	Point3D CA = m_vertexPoints[0] - m_vertexPoints[2];

	// Normals to edges of the triangle,
	// looking inside the triangle
	Point3D n1, n2, n3;
	// Normal to AB
	n1 = Point3D(AB.GetY(), (-1) * AB.GetX(), 0);
	if (Point3D::ScalarProduct(n1, BC) < 0)
		n1 = (-1) * n1;
	// Normal to BC
	n2 = Point3D(BC.GetY(), (-1) * BC.GetX(), 0);
	if (Point3D::ScalarProduct(n2, CA) < 0)
		n2 = (-1) * n2;
	// Normal to CA
	n3 = Point3D(CA.GetY(), (-1) * CA.GetX(), 0);
	if (Point3D::ScalarProduct(n3, AB) < 0)
		n3 = (-1) * n3;

	return (Point3D::ScalarProduct(point - m_vertexPoints[0], n1) > 0 &&
		    Point3D::ScalarProduct(point - m_vertexPoints[1], n2) > 0 &&
			Point3D::ScalarProduct(point - m_vertexPoints[2], n3) > 0 );
			


	
	/*
	if ( (point == m_vertexPoints[0]) ||
		 (point == m_vertexPoints[1]) ||
		 (point == m_vertexPoints[2]) )
	{
		return true;
	}

	real alpha1 = RealAbs( acos(Point3D::Angle(m_vertexPoints[1] - m_vertexPoints[0],
										   point - m_vertexPoints[0])) );
	real alpha2 = RealAbs( acos(Point3D::Angle(m_vertexPoints[2] - m_vertexPoints[0],
										   point - m_vertexPoints[0])) );

	real betha1 = RealAbs( acos(Point3D::Angle(m_vertexPoints[0] - m_vertexPoints[1],
									   point - m_vertexPoints[1])) );
	real betha2 = RealAbs( acos(Point3D::Angle(m_vertexPoints[2] - m_vertexPoints[1],
									   point - m_vertexPoints[1])) );
	real gamma1 = RealAbs( acos(Point3D::Angle(m_vertexPoints[0] - m_vertexPoints[2],
									   point - m_vertexPoints[2])) );
	real gamma2 = RealAbs( acos(Point3D::Angle(m_vertexPoints[1] - m_vertexPoints[2],
									   point - m_vertexPoints[2])) );

	return ( REAL_EQUAL(alpha1 + alpha2,
				RealAbs(acos(Point3D::Angle(m_vertexPoints[1] - m_vertexPoints[0],
								     m_vertexPoints[2] - m_vertexPoints[0])))) ) &&
		   ( REAL_EQUAL(betha1 + betha2,
				RealAbs(acos(Point3D::Angle(m_vertexPoints[0] - m_vertexPoints[1],
								     m_vertexPoints[2] - m_vertexPoints[1])))) ) &&
		   ( REAL_EQUAL(gamma1 + gamma2,
		     RealAbs(acos(Point3D::Angle(m_vertexPoints[1] - m_vertexPoints[2],
								     m_vertexPoints[0] - m_vertexPoints[2])))) ) ;
	*/
}

/// by Olga - this function returns true if the given
/// triangle is fully contained inside *this. 
/// Pre-condition: both triangle and *this are on xy plane
inline bool Triangle::ContainsTriangle(const Triangle & triangle)
{
	return ( ContainsPoint(triangle.GetFirstVertex()) &&
			 ContainsPoint(triangle.GetSecondVertex()) &&
			 ContainsPoint(triangle.GetThirdVertex()) );
}

/// by Olga
inline void Triangle::RotateClockwiseAboutVertex(USHORT vertexInd, real angle)
{
	assert(vertexInd < 3);

	Point3D rotationVertex = m_vertexPoints[vertexInd];

	Translate((-1)*rotationVertex);
	
	USHORT	ind2 = (vertexInd + 1) % 3;
	USHORT	ind3 = (vertexInd + 2) % 3;
	real	cs	 = cos(angle);
	real	sn	 = sin(angle);
	real	x2	 = m_vertexPoints[ind2].GetX();
	real	y2	 = m_vertexPoints[ind2].GetY();
	real	x3	 = m_vertexPoints[ind3].GetX();
	real	y3	 = m_vertexPoints[ind3].GetY();

	m_vertexPoints[ind2].SetX(cs * x2 + sn * y2);
	m_vertexPoints[ind2].SetY((-sn) * x2 + cs * y2);

	m_vertexPoints[ind3].SetX(cs * x3 + sn * y3);
	m_vertexPoints[ind3].SetY((-sn) * x3 + cs * y3);
	Translate(rotationVertex);
}

/// by Olga
inline void Triangle::RotateCounterClockwiseAboutVertex(USHORT vertexInd, real angle)
{
	assert(vertexInd < 3);

	Point3D rotationVertex = m_vertexPoints[vertexInd];

	Translate((-1)*rotationVertex);
	
	USHORT	ind2 = (vertexInd + 1) % 3;
	USHORT	ind3 = (vertexInd + 2) % 3;
	real	cs	 = cos(angle);
	real	sn	 = sin(angle);
	real	x2	 = m_vertexPoints[ind2].GetX();
	real	y2	 = m_vertexPoints[ind2].GetY();
	real	x3	 = m_vertexPoints[ind3].GetX();
	real	y3	 = m_vertexPoints[ind3].GetY();

	m_vertexPoints[ind2].SetX(cs * x2 - sn * y2);
	m_vertexPoints[ind2].SetY(sn * x2 + cs * y2);

	m_vertexPoints[ind3].SetX(cs * x3 - sn * y3);
	m_vertexPoints[ind3].SetY(sn * x3 + cs * y3);
	Translate(rotationVertex);
}

/// by Olga
inline const Point3D & Triangle::GetVertex(USHORT ind) const throw()
{
	assert(ind < 3);
	return m_vertexPoints[ind];
}

/// by Olga
inline Triangle Triangle::GetTranslated(const Point3D & translateVector) const throw()
{
	return Triangle(m_vertexPoints[0] + translateVector,
					m_vertexPoints[1] + translateVector,
					m_vertexPoints[2] + translateVector);
}

#endif // __TRIANGLE_H
