// VRMLLoader.cpp
// adi karol (aug 01)

#pragma warning( disable : 4786 )

// #define CHECK_VRML_LOADING



// #include "TM.h"
#include "TexturedMesh.h"

#include "wrlFlexLexer.h"
#include "x3dExporter.h"
#include "element.h"
#include "attribute.h"

#include <fstream.h>
#ifdef _MSC_VER
#include <strstrea.h>
#else
#include <strstream.h>
#endif


#if defined(_DEBUG) && 0
#define RETURN_FALSE assert(false)
#else
#define RETURN_FALSE return false
#endif



class vrmlException {
} s_vrmlException;

bool vrml_setFaces(TexturedMesh *tm, const char *cszCoordIndex) {
	char *szCoordIndex;
	int ind1, ind2, ind3, ind;
	
	if (!(szCoordIndex = new char[strlen(cszCoordIndex) + 1]))
		RETURN_FALSE;
	strcpy(szCoordIndex, cszCoordIndex);
	
	try {
		
		istrstream sIn(szCoordIndex);
		while (!sIn.fail()) {
			ind1 = ind2 = ind3 = -1;
			ind = -1;	// trailing -1 is optional
			sIn >> ind1;
			if (sIn.fail()) break;
			sIn>> ind2 >> ind3 >> ind;
			if (ind1 < 0 || ind2 < 0 || ind3 < 0 || ind != -1)
				throw s_vrmlException;
			tm->m_faces.push_back(Tripplet(ind1, ind2, ind3));
		}
		
	} catch (vrmlException) {
		delete[] szCoordIndex;
		RETURN_FALSE;
	}
	delete[] szCoordIndex;
	return true;
}

bool vrml_setTexFaces(TexturedMesh *tm, const char *cszTexCoordIndex) {
	char *szTexCoordIndex;
	int ind1, ind2, ind3, ind;
	
	if (!(szTexCoordIndex = new char[strlen(cszTexCoordIndex) + 1]))
		RETURN_FALSE;
	strcpy(szTexCoordIndex, cszTexCoordIndex);
	
	try {
		
		istrstream sIn(szTexCoordIndex);
		while (!sIn.fail()) {
			ind1 = ind2 = ind3 = -1;
			ind = -1;	// trailing -1 is optional
			sIn >> ind1;
			if (sIn.fail()) break;
			sIn>> ind2 >> ind3 >> ind;
			if (ind1 < 0 || ind2 < 0 || ind3 < 0 || ind != -1)
				throw s_vrmlException;
			tm->m_uv_per_triangle.push_back(Tripplet(ind1, ind2, ind3));
		}
		
	} catch (vrmlException) {
		delete[] szTexCoordIndex;
		RETURN_FALSE;
	}
	delete[] szTexCoordIndex;
	return true;
}

bool vrml_setPoints(TexturedMesh *tm, const char *cszPoint) {
	char *szPoint;
	real x, y, z;
	if (!(szPoint = new char[strlen(cszPoint) + 1]))
		RETURN_FALSE;
	strcpy(szPoint, cszPoint);
	
	try {
		istrstream sIn(szPoint);
		while (sIn.good()) {
			sIn >> x >> y >> z;
			tm->m_vertices.push_back(Point3D(x, y, z));
		}
		
	} catch (vrmlException) {
		delete[] szPoint;
		RETURN_FALSE;
	}
	
	delete[] szPoint;
	return true;
}

bool vrml_setUVPoints(TexturedMesh *tm, const char *cszPoint) {
	char *szPoint;
	real u, v;
	if (!(szPoint = new char[strlen(cszPoint) + 1]))
		RETURN_FALSE;
	strcpy(szPoint, cszPoint);
	
	try {
		istrstream sIn(szPoint);
		while (sIn.good()) {
			sIn >> u >> v;
			tm->m_uv_coordinates.push_back(Point2D(u, v));
		}
		
	} catch (vrmlException) {
		delete[] szPoint;
		RETURN_FALSE;
	}
	
	delete[] szPoint;
	return true;
}

bool vrml_setUrl(TexturedMesh *tm, const char *cszUrl) {
	char *szUrl;
	if (!(szUrl = new char[strlen(cszUrl) + 1]))
		RETURN_FALSE;
	strcpy(szUrl, cszUrl);
	tm->m_textures.push_back(szUrl);
	return true;
}



#ifdef _MSC_VER
ofstream m_fNull("nul:");
#else
ofstream m_fNull("/dev/null");
#endif
x3dExporter m_exporter(m_fNull);


inline int operator==(const Element &element, const char *tag) {
	return !strcmp(element.getTag()->c_str(), tag);
}

inline int operator==(const Attribute &attribute, const char *name) {
	return !strcmp(attribute.getName()->c_str(), name);
}

static TexturedMesh *s_texturedMesh;

bool processElement(const Element *element) {
	static bool s_shape;
	static bool s_geometry;
	static bool s_appearance;
	
	if (*element == "X3D") {
		// X3D: initialization
		s_shape = false;
		s_geometry = false;
		s_appearance = false;
	} else if (*element == "Shape") {
		// Shape: begin textured mesh
		// currently, only the first shape is extracted
		s_shape = true;
	} else if (s_shape) {
		if (*element == "geometry") {
			// Shape/geometry: begin geometry
			s_geometry = true;
		} else if (*element == "appearance") {
			// Shape/appearance: begin appearance
			s_appearance = true;
		} else if (s_geometry) {
			if (*element == "IndexedFaceSet") {
				// Shape/geometry/IndexedFaceSet
				for (Element::AttributeList::const_iterator attrIter = element->getAttributes().begin(); attrIter !=
					element->getAttributes().end(); attrIter++) {
					if (**attrIter == "coordIndex") {
						// Shape/geometry/IndexedFaceSet/@coordIndex
						const char *cszCoordIndex = (*attrIter)->getValue()->c_str();
						if (!vrml_setFaces(s_texturedMesh, cszCoordIndex))
							RETURN_FALSE;
					}
					if (**attrIter == "texCoordIndex") {
						// Shape/geometry/IndexedFaceSet/@texCoordIndex
						const char *cszTexCoordIndex = (*attrIter)->getValue()->c_str();
						if (!vrml_setTexFaces(s_texturedMesh, cszTexCoordIndex))
							RETURN_FALSE;
					}
				}
			} else if (*element == "Coordinate") {
				// Shape/geometry/*/Coordinate
				for (Element::AttributeList::const_iterator attrIter = element->getAttributes().begin(); attrIter !=
					element->getAttributes().end(); attrIter++) {
					if (**attrIter == "point") {
						// Shape/geometry/*/Coordinate/@point
						const char *cszPoint = (*attrIter)->getValue()->c_str();
						if (!vrml_setPoints(s_texturedMesh, cszPoint))
							RETURN_FALSE;
					}
				}
			} else if (*element == "TextureCoordinate") {
				// Shape/geometry/*/TextureCoordinate
				for (Element::AttributeList::const_iterator attrIter = element->getAttributes().begin(); attrIter !=
					element->getAttributes().end(); attrIter++) {
					if (**attrIter == "point") {
						// Shape/geometry/*/TextureCoordinate/@point
						const char *cszPoint = (*attrIter)->getValue()->c_str();
						if (!vrml_setUVPoints(s_texturedMesh, cszPoint))
							RETURN_FALSE;
					}
				}
			}
			
		} else if (s_appearance) {
			if (*element == "ImageTexture") {
				// Shape/appearance/*/ImageTexture
				/// need multi-textured meshes examples for extending to such case
				for (Element::AttributeList::const_iterator attrIter = element->getAttributes().begin(); attrIter !=
					element->getAttributes().end(); attrIter++) {
					if (**attrIter == "url") {
						// Shape/appearance/*/ImageTexture/@url
						const char *cszUrl = (*attrIter)->getValue()->c_str();
						if (!vrml_setUrl(s_texturedMesh, cszUrl))
							RETURN_FALSE;
					}
				}
				
			}
		}
		
	}
	
	for (Element::ElementList::const_iterator elemIter = element->getElements().begin(); elemIter !=
		element->getElements().end(); elemIter++) {
		if (!processElement(*elemIter))
			RETURN_FALSE;
	}	
	
	if (*element == "appearance") {
		// end appearance
		s_appearance = false;
	} else if (*element == "geometry") {
		// end geometry
		s_geometry = false;
	} else if (*element == "Shape") {
		// end Shape
		// currently, only the first shape is extracted
		s_shape = false;
		return true;
	}
	
	return true;
}


bool TexturedMesh::ReadFromVRML(ifstream &fIn) {
	if (fIn.fail())
		RETURN_FALSE;
	
	extern int yyparse();
	extern Element *vrmlRoot;
	
	wrlFlexLexer flexLexer(&fIn);
	flexLexer.m_lineno = 1;
	flexLexer.set_debug(0);
	
	try {
		if (yyparse())
			throw -1;	// should not occur, since wrlFlexLexer::yyError()
		// throws line number
	} catch (int lineno) {
		// lineno is line # in vrml file where error occurred
		lineno = lineno;	// unreferenced
		RETURN_FALSE;		// lexer/parser error
	}
	
	s_texturedMesh = this;
	if (!processElement(vrmlRoot))
		RETURN_FALSE;		// mesh extraction error
	
	if (!assertValid())
		RETURN_FALSE;		// mesh internal state error


// Added by Olga - dimension check
	if (IsMesh2D())
		m_is2Dmesh = true;
	else
		m_is2Dmesh = false;
// End changes

	return true;
}


bool TexturedMesh::assertValid() const
{
/*	
	{ // check m_textures
		if (m_textures.size() < 1)
			RETURN_FALSE;
	}
*/	
	{ // check m_faces indices
		int max = m_vertices.size() - 1;
		for (std::vector<Tripplet>::const_iterator it = m_faces.begin(); it != m_faces.end(); it++)
			if ((*it).GetFirstIndex() > max || (*it).GetSecondIndex() > max || (*it).GetThirdIndex() > max)
				RETURN_FALSE;
	}
	
	{ // check m_uv_per_triangle
		int max = m_uv_coordinates.size() - 1;
		for (std::vector<Tripplet>::const_iterator it = m_faces.begin(); it != m_faces.end(); it++)
			if ((*it).GetFirstIndex() > max || (*it).GetSecondIndex() > max || (*it).GetThirdIndex() > max)
				RETURN_FALSE;
	}
	
	return true;
}






/**********************************/

#ifdef CHECK_VRML_LOADING

class TexturedMesh2 : public TexturedMesh {
public:
	void dump() {
		cout << "points: " << m_vertices.size() << endl;
		cout << "faces: " << m_faces.size() << " == " << m_uv_per_triangle.size() << endl;
		cout << "uv pairs: " << m_uv_coordinates.size() << endl;
		cout << "textures: " << m_textures.size() << endl;
	}
};



int main(int argc, char **argv) 
{
/*
	ifstream fVrml(argc >= 2 ? argv[1] : "Hedra4texture.WRL");
	TexturedMesh2 tm;
	if (!fVrml.fail()) {
		if (!tm.ReadFromVRML(fVrml))
			cout << "error reading VRML file" << endl;
		else {
			cout << "success" << endl;
			tm.dump();
		}
		
		fVrml.close();

		ofstream out_file("HedraTex.wrl");
		tm.WriteToVRML(out_file, true);
		out_file.close();

	}

	
	ifstream fVrml2(argc >= 3 ? argv[2] : "Tube.WRL");
	TexturedMesh2 tm2;
	if (!fVrml2.fail()) {
		if (!tm2.ReadFromVRML(fVrml2))
			cout << "error reading VRML file" << endl;
		else {
			cout << "success" << endl;
			tm2.dump();
		}
		
		fVrml2.close();
	}

	ofstream out_file2("TubeTex.wrl");
	tm2.WriteToVRML(out_file2, true);
	out_file2.close();
*/

	ifstream fVrml(argc >= 2 ? argv[1] : "patch.WRL");
	TexturedMesh tm;
	if (!fVrml.fail()) 
	{
		if (!tm.ReadFromVRML(fVrml))
			cout << "error reading VRML file" << endl;
		else {
			cout << "success" << endl;
		}
		
		fVrml.close();
/*
		ofstream out_file("patch_out.wrl");
		tm.WriteToVRML(out_file, true);
		out_file.close();
*/
		tm.Render2DMeshToBMP("patch_rendered_tex.bmp", true);
		tm.Render2DMeshToBMP("patch_rendered_wire.bmp", false);

		std::vector<TexturedMesh> mesh_vec;
		tm.Flatten(mesh_vec, 1.0);

/*
		for (int i = 0; i < mesh_vec.size(); ++i)
		{
			char filename[50] = "";
			sprintf(filename, "copy_patch_tex_%d_%d.bmp", i, mesh_vec[i].GetNumberOfTriangles());
			mesh_vec[i].Render2DMeshToBMP(filename, true);
		}
*/
		ofstream out_vrml_charts("Patch_charts.wrl");
		tm.WriteToVRML(out_vrml_charts, true);
		out_vrml_charts.close();
	}

	return 0;
}


#endif // CHECK_VRML_LOADING
