#ifndef GTB_BOX3_INCLUDED
#define GTB_BOX3_INCLUDED

#include "gtbmousecommon.h"
#include "point3.hpp"

BEGIN_GTBMOUSENS


class Vector3;
class Plane;
class Polygon3;


class Box3 {
public:
	enum { INSIDE, INTERSECT, OUTSIDE };

	enum Face { L, R, D, U, B, F };

	enum Vertex { LDB,
		      LDF,
		      LUB,
		      LUF,
		      RDB,
		      RDF,
		      RUB,
		      RUF
	};

	enum VertexMask {
		RIGHT_MASK =	1 << 2,
		UP_MASK =	1 << 1,
		FRONT_MASK =	1 << 0
	};

	Box3();
	Box3(const Box3 &b);
	Box3(const Point3 &min_pt, const Point3 &max_pt);
	Box3(real_t x_min, real_t y_min, real_t z_min,
	     real_t x_max, real_t y_max, real_t z_max);
	Box3(const Point3 &centroid,
	     real_t x_length,
	     real_t y_length, 
	     real_t z_length);
	Box3 &operator=(const Box3 &b);

	bool operator==(const Box3 &b) const;
	bool operator!=(const Box3 &b) const;
	bool is_empty() const;

	const Point3 &min_point() const;
	const Point3 &max_point() const;

	real_t x_min() const;
	real_t y_min() const;
	real_t z_min() const;

	real_t x_max() const;
	real_t y_max() const;
	real_t z_max() const;

	void reset(const Point3 &min_pt, const Point3 &max_pt);
	void reset(real_t x_min, real_t y_min, real_t z_min,
		   real_t x_max, real_t y_max, real_t z_max);
	void scale(real_t s);

	void set_min(int axis, double value);
	void set_max(int axis, double value);

	real_t x_length() const;
	real_t y_length() const;
	real_t z_length() const;

	real_t diagonal_length() const;
	real_t shortest_axis_length() const;
	real_t longest_axis_length() const;

	real_t volume() const;

	Point3 centroid() const;
	bool contains(const Point3 &p) const;
	int classify_position(const Box3 &b) const;
	Point3 vertex(unsigned i) const; // i < 8
	Point3 vertex(unsigned face, unsigned i) const; // i < 4
	Vector3 normal(unsigned i) const; // i < 6
	Polygon3 face(unsigned i) const; // i < 6
	Plane plane(unsigned i) const; // i < 6

	void render() const;
	void outline() const;

	void read(FILE *fp);
	void write(FILE *fp) const;

	static Box3 bounding_box(const Point3 &a,
				 const Point3 &b,
				 const Point3 &c);
	static Box3 bounding_box(const std::vector<Point3> &v);
	static Box3 make_union(const Box3 &b1, const Box3 &b2);

	bool is_visible() const;

	double distance(const Point3& p) const;

protected:
	bool is_order_correct() const;

	Point3 _min_pt, _max_pt;
	static const unsigned _vertex_indices[6][4];
};


END_GTBMOUSENS

#ifndef OUTLINE
#include "box3.ipp"
#endif

#endif // GTB_BOX3_INCLUDED
