#pragma warning( disable : 4786 )

#include "TexturedMesh.h"
#include <iostream.h>
#include <time.h>

void WriteLog(char *model_filename,
			  real max_distortion,
			  real min_crease_angle,
			  real crease_alpha,
			  real max_perimeter_to_area_ratio,
			  bool allow_cracks,
			  int max_vertices,
			  bool prepartition,
			  int num_patches,
			  int minutes,
			  int seconds,
			  real av_distortion = 0);

void main()
{
	char vrml_filename[100] = "";
	
	cout << "Enter filename (*.wrl) : ";
	cin >> vrml_filename;

	ifstream fVrml(vrml_filename, ios::in|ios::nocreate);

	TexturedMesh tm;
	if (!fVrml.fail()) 
	{

		if (!tm.ReadFromVRML(fVrml))
		{
			cout << "error reading VRML file" << endl;
			fVrml.close();
			return;
		}
		else {
			cout << "success reading VRML file" << endl;
		}

		fVrml.close();

		cout << "The model:\n";
		cout << "\tnumber of vertices: " << tm.GetNumberOfVertices() << endl;
		cout << "\tnumber of triangles: " << tm.GetNumberOfTriangles() << endl << endl;

		ofstream outF("out.wrl");
		tm.WriteToVRML(outF, false);
		outF.close();
	}
	else
	{
		cout << "error reading VRML file" << endl;
	}

}

void WriteLog(char *model_filename,
			  real max_distortion,
			  real min_crease_angle,
			  real crease_alpha,
			  real max_perimeter_to_area_ratio,
			  bool allow_cracks,
			  int max_vertices,
			  bool prepartition,
			  int num_patches,
			  int minutes,
			  int seconds,
			  real av_distortion)
{
	ofstream log_file("ResultsLog.txt");

	log_file << "Model filename: " << model_filename << endl << endl;
	log_file << "Parameters:" << endl;
	log_file << "\tmax_distortion = " << max_distortion << endl;
	log_file << "\tmin_crease_angle = " << min_crease_angle << endl;
	log_file << "\tcrease_alpha = " << crease_alpha << endl;
	log_file << "\tmax_perimeter_to_area_ratio = " << max_perimeter_to_area_ratio << endl;
	log_file << "\tallow cracks = " << allow_cracks << endl;
	log_file << "\tmax vertices per patch = " << max_vertices << endl;
	log_file << "\tprepartition by normals = " << prepartition << endl;

	log_file << endl;

	log_file << "Results:" << endl;
	log_file << "\tNumber of patches: " << num_patches << endl;
	log_file << "\tFlattening took " << minutes << " minutes and " << seconds << " seconds";
#ifdef  _DEBUG
	log_file << " in Debug mode. " << endl;
#else
	log_file << " in Release mode. " << endl;
#endif

	log_file.close();
}

