#ifndef GTB_MATRIX4_INCLUDED
#define GTB_MATRIX4_INCLUDED


#include "gtbmousecommon.h"
#include <iostream>

BEGIN_GTBMOUSENS


class Matrix4 {
public:
	Matrix4();

	explicit Matrix4(float m[4][4]);
	explicit Matrix4(float m[16]);

	explicit Matrix4(double m[4][4]);
	explicit Matrix4(double m[16]);

	Matrix4(real_t m00, real_t m01, real_t m02, real_t m03,
		real_t m10, real_t m11, real_t m12, real_t m13,
		real_t m20, real_t m21, real_t m22, real_t m23,
		real_t m30, real_t m31, real_t m32, real_t m33);

	Matrix4(const Matrix4 &m);
	Matrix4 &operator=(const Matrix4 &m);

	bool operator==(const Matrix4 &m) const;
	bool operator!=(const Matrix4 &m) const;

	Matrix4 &make_identity();
	Matrix4 &negate();

	Matrix4 transpose() const;
	Matrix4 inverse() const;
	real_t det() const;

	real_t *operator[](unsigned i);
	const real_t *operator[](unsigned i) const;

	const real_t *as_array() const;
	real_t *as_array();

	void print(FILE *fp) const;
	void scan(FILE *fp);

	Matrix4 operator-();
	Matrix4 &operator+=(const Matrix4 &m);
	Matrix4 &operator-=(const Matrix4 &m);
	Matrix4 &operator*=(const Matrix4 &m);
	Matrix4 &operator*=(real_t a);

	friend Matrix4 operator+(const Matrix4 &m1, const Matrix4 &m2);
	friend Matrix4 operator-(const Matrix4 &m1, const Matrix4 &m2);
	friend Matrix4 operator*(const Matrix4 &m1, const Matrix4 &m2);
	friend Matrix4 operator*(const real_t a, const Matrix4 &m);
	friend Matrix4 operator*(const Matrix4 &m, real_t a);

	friend std::ostream &operator<<(std::ostream &out, const Matrix4 &m);

protected:
	bool LU_decomposition(int p[4]);

	void LU_back_substitution(const int p[4],
				  Matrix4 &b,
				  Matrix4 &x);

	real_t _m[4][4];
};


extern const Matrix4 MATRIX4_ZERO;
extern const Matrix4 MATRIX4_IDENTITY;


END_GTBMOUSENS

#ifndef OUTLINE
#include "matrix4.ipp"
#endif

#endif // GTB_MATRIX4_INCLUDED
