#include <gtbmouse.h>
using namespace gtbmouse;

static Mouse _mouse;
static View _view;
static Point3 _origin;
static double _scale_factor;

void init()
{
    _view.init_exterior_view(_mesh->bounding_box());
    _scale_factor = _mesh->bounding_box().longest_axis_length() * 0.01;
}

static void display_cb()
{
    //
    // Render
    //
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    _view.load_camera();

    if (lighing_enabled) {
        glEnable(GL_LIGHTING);
    } else {
        glDisable(GL_LIGHTING);
    }

    Render();

    glutSwapBuffers();

}


static void reshape_cb(int w, int h)
{
    glViewport(0, 0, w, h);
    _view.set_viewport(Viewport(0, 0, w, h));
}

static void keyboard_cb(unsigned char key, int x, int y)
{
    int modifiers =  glutGetModifiers();

    bool redraw = false;
    switch(key) {
    case '.':
        {
            double modelmat[16];
            double projmat[16];
            int viewport[4];
            glGetDoublev(GL_MODELVIEW_MATRIX, modelmat);
            glGetDoublev(GL_PROJECTION_MATRIX, projmat);
            glGetIntegerv(GL_VIEWPORT, viewport);

            float depth;
            double wx = _mouse.x();
            double wy = viewport[3] - _mouse.y();
            glReadPixels(
                (int)wx, (int)wy,
                1, 1,
                GL_DEPTH_COMPONENT,
                GL_FLOAT,
                &depth);
            printf(" '.' depth: %2.10g\n", depth);

            if (depth < 0.999999f)
            {
                double x_,y_,z_;
                gluUnProject(wx, wy, depth, modelmat, projmat, viewport, &x_, &y_, &z_);
                _origin.reset(x_,y_,z_);
                printf(" Set new origin: %g %g %g\n", x_, y_, z_);
            }
            break;
        }
        break;
    case 27:
		// Close Matlab
		engClose(ep);
        exit(EXIT_SUCCESS);
        break;
    default:
        keys();
        break;
    }
    if (redraw) {
        glutPostRedisplay();
    }
}

static void special_cb(int key, int x, int y)
{
    (void)x;
    (void)y;

    bool redraw = false;
    int modifiers =  glutGetModifiers();

    const char *camera_names[] = 
    {
        "camera1.cam",
        "camera2.cam",
        "camera3.cam",
        "camera4.cam"
    };

    const char *texture_names[] = 
    {
        "txt1.txp",
        "txt2.txp",
        "txt3.txp",
        "txt4.txp"
    };

    switch(key)
    {
    case GLUT_KEY_F1:
    case GLUT_KEY_F2:
    case GLUT_KEY_F3:
    case GLUT_KEY_F4:
    case GLUT_KEY_F5:
    case GLUT_KEY_F6:
        {
            const char* name = camera_names[key - GLUT_KEY_F1];
            const char* txtname = texture_names[key - GLUT_KEY_F1];
            if (modifiers & GLUT_ACTIVE_ALT)
            {
                _view.camera().read(name);
                viewer->load_texture_params(txtname);
                redraw = true;
            }
            else
            {
                _view.camera().write(name);
                viewer->save_texture_params(txtname);
            }
        }
        break;
    default:
        keys();
        break;
    }

    if (redraw) {
        glutPostRedisplay();
    }
}


static void mouse_cb(int button, int state, int x, int y)
{
    _mouse.set_button_state(button, state);
    _mouse.set_position(x, y);
}


static void passive_motion_cb(int x, int y)
{
    _mouse.set_position(x, y);
}

static void motion_cb(int x, int y)
{
    _mouse.set_position(x, y);
    bool redraw = false;
    if (_mouse.is_button_pressed(GLUT_LEFT_BUTTON)) {
        int dx = _mouse.dx();
        int dy = _mouse.dy();
        if (keyboard::is_shift_pressed()) {
            _view.translate_world(_origin, dx, dy);
            redraw = true;
        } else if (keyboard::is_control_pressed()) {
            _view.scale_world(_scale_factor * dy);
            redraw = true;
        } else {
            _view.rotate_world(_origin, dx, dy);
            redraw = true;
        }
    }
    if (redraw) {
        glutPostRedisplay();
    }
}

