#pragma warning( disable : 4786 )
#pragma warning( disable : 4503 )


#include "TexturedMesh.h"
#include <string.h>
#include <iostream>
#include <fstream> 
#include <stdio.h>
#include <stdlib.h>

void printUsage();
bool LoadMesh(char *filename, TexturedMesh & mesh);

// Allocates new string that needs to be freed
char * RemoveExtension(char *filename);

void printUsage() {
	cout << "****************************************************************************************" << endl;
	cout << "This program reads meshes in ply2 format, solves the delta-system and xyz-quantizes\n";
	cout << "Usage:" << endl;
	cout << "mparser <filename> [options]" << endl;
	cout << "    Options:" << endl;
	cout << "      -db <delta_bits_x> <delta_bits_y> <delta_bits_z>" << endl;
	cout << "        Generates delta-quantized mesh. If all delta_bits==-1 then no quant. (for debugging)" << endl;
	cout << "      -qb <xyz_bits>" << endl;
	cout << "        Generates xyz-quantized mesh" << endl;
	cout << "      -a <num_anchor_iterations>" << endl;
	cout << "        Iterates and adds num_anchor_iterations anchors" << endl;
	cout << "      -la <num_anch>" << endl; 
	cout << "        Loads anchors from file, uses the first num_anch. If num_anch < 0" << endl;
	cout << "        or num_anch > number of anchors in the file, uses all the anchors" << endl;
	cout << "      -sa" << endl; 
	cout << "        Saves the anchors to file" << endl;
	cout << "      -taucslog" << endl;
	cout << "        Outputs taucs messages on stdout" << endl;
	cout << "****************************************************************************************" << endl;
}

char * RemoveExtension(char *filename) {
	char *ep = strstr(filename, ".");
	int length = ep - filename + 1;
	
	char * ret = new char[length];
	memcpy(ret, filename, length*sizeof(char));
	ret[length - 1] = '\0';
	
	return ret;
}


bool LoadMesh(char *filename, TexturedMesh & mesh) {
	if (strlen(filename) <= 0) {
		return false;
	}
	
	ifstream inFile(filename, ios::in|ios::nocreate);
	
	if (inFile.fail()) {
		return false;
	}
	
	if (strcmp(strstr(filename, "."), ".smpl") == 0) {
		mesh.ReadFromSimpleFormat(inFile);
		return true;
	}
	else if (strcmp(strstr(filename, "."), ".ply2") == 0)	{
		return mesh.ReadFromPly2Format(inFile);
	}
	/*
	else if (strcmp(strstr(filename, "."), ".wrl") == 0)	{
    return mesh.ReadFromVRML(inFile);
	}
	*/
	else {
		return false;
	}
}

int main(int argc, char *argv[]) {
	if (argc < 2) {
		printUsage();
		return 0;
	}
	
	TexturedMesh mesh;
	if (! LoadMesh(argv[1], mesh)) {
		cerr << "Couldn't read the mesh from " << argv[1] << endl;
		return -1;
	}
	
	cout << "before cleanup:" << endl;
	cout << "num vertices: " << mesh.GetNumberOfVertices() << endl;
	cout << "num triangles: " << mesh.GetNumberOfTriangles() << endl;
	
	// cout << "no. of boundary vertices: " << mesh.GetNumberOfBoundaryVertices() << endl;
	
	/*
	Point3D cent = mesh.GetCenterOfMass();
	mesh.Translate(-1.0 * cent);
	
	 // scale mesh according to view
	 Box bb = mesh.GetBoundingBox();
	 real max_edge = bb.GetMaxEdgeLength();
	 
	  mesh.Scale(1.0/max_edge);
	*/
	
	/*
	mesh.CleanIsolatedVertices();
	
	 cout << "after cleanup:" << endl;
	 cout << "num vertices: " << mesh.GetNumberOfVertices() << endl;
	 cout << "num triangles: " << mesh.GetNumberOfTriangles() << endl;
	*/  
	
	bool do_delta = false;
	bool do_xyz_quant = false;
	bool load_anchors = false;
	bool save_anchors = false;
	
	int delta_bits_x = 0;
	int delta_bits_y = 0;
	int delta_bits_z = 0;
	int xyz_bits = 0;
	int num_anchor_iterations = 0;
	int num_anchors_2load = 0;
	
	// parsing the options
	int opt_num = 2;
	while (opt_num < argc) {
		if (strcmp(argv[opt_num], "-taucslog") == 0) {
			taucs_logfile("stdout");
		}
		if (strcmp(argv[opt_num], "-db") == 0) {
			if (argc - opt_num - 1 < 3) {
				cerr << "You need to provide delta_bits{x,y,z}" << endl;
				return -2;
			}
			do_delta = true;
			delta_bits_x = atoi(argv[++opt_num]);
			delta_bits_y = atoi(argv[++opt_num]);
			delta_bits_z = atoi(argv[++opt_num]);
		}
		else if (strcmp(argv[opt_num], "-qb") == 0) {
			if (argc - opt_num - 1 < 1) {
				cerr << "You need to provide xyz_bits" << endl;
				return -3;
			}
			do_xyz_quant = true;
			xyz_bits = atoi(argv[++opt_num]);
		}
		else if (strcmp(argv[opt_num], "-a") == 0) {
			if (argc - opt_num - 1 < 1) {
				cerr << "You need to provide number of anchor iterations" << endl;
				return -4;
			}
			num_anchor_iterations = atoi(argv[++opt_num]);
		}
		else if (strcmp(argv[opt_num], "-la") == 0) {
			if (argc - opt_num - 1 < 1) {
				cerr << "You need to provide number of anchors to load" << endl;
				return -5;
			}
			load_anchors = true;
			num_anchors_2load = atoi(argv[++opt_num]);
		}
		else if (strcmp(argv[opt_num], "-sa") == 0) {
			save_anchors = true;
		}
		++opt_num;
	}
	
	char *filename_noext = RemoveExtension(argv[1]);
	
	char log_filename[100];
	sprintf(log_filename, "results/log_%s.txt", filename_noext);
	ofstream logFile(log_filename, ios::out|ios::app);
	
	logFile << "\n\nRan with the following arguments:" << endl;
	for (int i=0; i < argc; ++i) {
		logFile << argv[i] << " ";
	}
	logFile << endl;
	
	if (do_delta) {
		if (load_anchors) {
			char anch_filename[100];
			sprintf(anch_filename, "results/anchors_%s_%d_%d_%d.hnd", 
				filename_noext,
				delta_bits_x,
				delta_bits_y,
				delta_bits_z);
			ifstream anch_in_file(anch_filename);
			if (! anch_in_file) {
				cerr << "Failed to open anchors file " << anch_filename << endl;
				num_anchors_2load = 0;
			} 
			else {
				mesh.ReadAnchorsFromFile(anch_in_file, num_anchors_2load);
			}
			anch_in_file.close();
		}
		
		bool ret = mesh.ComputeDeltaQuantizedGeometry(delta_bits_x,
			delta_bits_y,
			delta_bits_z,
			num_anchor_iterations);
		if (! ret){
			cout << "Failed to solve the system, no file will be outputted for delta" << endl;
		}
		
		char deltafilename[100];
		sprintf(deltafilename, "results/deltares_%s_%d_%d_%d_%d.ply2", 
			filename_noext,
			delta_bits_x,
			delta_bits_y,
			delta_bits_z,
			mesh.GetNumberOfAnchors());
		
		ofstream outFileD(deltafilename);
		
		if (! outFileD) {
			cerr << "Failed to open " << deltafilename << " for writing!" << endl;
		}
		else {
			mesh.WriteToPly2Format(outFileD, 1); // write the delta-geometry
		}
		outFileD.close();
		
		if (save_anchors) {
			char anch_filename[100];
			sprintf(anch_filename, "results/anchors_%s_%d_%d_%d.hnd", 
				filename_noext,
				delta_bits_x,
				delta_bits_y,
				delta_bits_z);
			ofstream anch_out_file(anch_filename);
			if (! anch_out_file) {
				cerr << "Failed to open anchors file " << anch_filename << " for writing" << endl;
			}
			else {
				mesh.WriteAnchorsToFile(anch_out_file);
			}
			anch_out_file.close();
		}
		
		if (ret) {
			float delta_entr = mesh.DeltaEntropy(delta_bits_x ,delta_bits_y, delta_bits_z);
			float anch_entr = mesh.OneAnchorEntropy(36);
			taucsType Mqrms = mesh.GetMqRMS(0);
			taucsType Sqrms = mesh.GetSqRMS(0);
			
			logFile << "\nDelta bits: "<<delta_bits_x<<" "<<delta_bits_y<<" "<<delta_bits_z<< endl;
			logFile << "Num anchors: " << mesh.GetNumberOfAnchors() << endl;
			logFile << "Pure delta entropy: " << delta_entr << endl;
			logFile << "Every anchor adds " << anch_entr << " to entropy" << endl;
			logFile << "Total entropy with anchors: " << delta_entr + mesh.GetNumberOfAnchors()*anch_entr << endl;
			logFile << "delta_Mq_rms: " << Mqrms << endl;
			logFile << "delta_Sq_rms: " << Sqrms << endl;
			logFile << "delta_visual_rms(0.15): " << Mqrms*0.15 + Sqrms*0.85 << endl; 
		}
	}
	
	if (do_xyz_quant) {
		char xyz_filename[100];
		sprintf(xyz_filename, "results/quantres_%s_%d.ply2", 
			filename_noext,
			xyz_bits);
		
		ofstream outFileQ(xyz_filename);
		
		if (! outFileQ) {
			cerr << "Failed to open " << xyz_filename << " for writing!" << endl;
		}
		else {
			mesh.ComputeXYZQuantizedGeometry(xyz_bits, xyz_bits, xyz_bits);
			mesh.WriteToPly2Format(outFileQ, 2); // write the xyz-quantized geometry
		}
		outFileQ.close();
		
		taucsType Mqrms = mesh.GetMqRMS(1);
		taucsType Sqrms = mesh.GetSqRMS(1);
		logFile << "\nXYZ quantization bits: " << xyz_bits << endl;
		logFile << "xyz_Mq_rms: " << Mqrms << endl;
		logFile << "xyz_Sq_rms: " << Sqrms << endl;
		logFile << "xyz_visual_rms(0.15): " << Mqrms*0.15 + Sqrms*0.85 << endl; 
	}
	
	delete [] filename_noext;
	
	return 0;
}


int old_main3(int argc, char *argv[]) {
	taucs_logfile("stdout");
	
	switch (argc) {
	case 2:
	case 3:
	case 4:
	case 5:
	case 6:
		cerr << "Not enough parameters" << endl;
		printUsage();
		return -1;
		break;
	case 7: // this is what we want: <progname> <filename> d_bits_x d_bits_y d_bits_z q_bits num_iterated anchors
		break;
	case 8:
	default:
		cerr << "Too many parameters!" << endl;
		printUsage();
		return -2;
		break;
	}
	
	// open the input file
	ifstream inFile(argv[1]);
	
	if (! inFile) {
		cerr << "Couldn't open file " << argv[1] << endl;
		return -3;
	}
	
	// reading the mesh
	TexturedMesh mesh;
	
	if (! mesh.ReadFromPly2Format(inFile)) {
		cerr << "Failed to read the mesh from supplied ply2 file" << endl;
		return -4;
	}
	
	cout << "before cleanup:" << endl;
	cout << "num vertices: " << mesh.GetNumberOfVertices() << endl;
	cout << "num triangles: " << mesh.GetNumberOfTriangles() << endl;
	
	// cout << "no. of boundary vertices: " << mesh.GetNumberOfBoundaryVertices() << endl;
	
	/*
	Point3D cent = mesh.GetCenterOfMass();
	mesh.Translate(-1.0 * cent);
	
	 // scale mesh according to view
	 Box bb = mesh.GetBoundingBox();
	 real max_edge = bb.GetMaxEdgeLength();
	 
	  mesh.Scale(1.0/max_edge);
	*/
	
	/*
	mesh.CleanIsolatedVertices();
	
	 cout << "after cleanup:" << endl;
	 cout << "num vertices: " << mesh.GetNumberOfVertices() << endl;
	 cout << "num triangles: " << mesh.GetNumberOfTriangles() << endl;
	*/  
	
	int delta_bits_x = atoi(argv[2]);
	int delta_bits_y = atoi(argv[3]);
	int delta_bits_z = atoi(argv[4]);
	int xyz_bits = atoi(argv[5]);
	int num_anchors = atoi(argv[6]);
	
	if (delta_bits_x != 0 && delta_bits_y != 0 && delta_bits_z != 0) {
		bool ret = mesh.ComputeDeltaQuantizedGeometry(delta_bits_x,delta_bits_y,delta_bits_z,num_anchors);
		if (! ret){
			cout << "Failed to solve the system, no file will be outputted for delta" << endl;
			return -17;
		}
		
		char *newname = new char[strlen(argv[1]) + 100];
		char *noext = new char[strlen(argv[1]) + 1];
		if (*(strstr(argv[1], ".ply2"))) {
			strcpy(noext, argv[1]);
			noext[strlen(argv[1]) - 5] = '\0';
		}
		else {
			strcpy(noext, argv[1]);
		}
		
		sprintf(newname, "results/deltares_%s_%d_%d_%d_%d.ply2", noext,
			delta_bits_x,
			delta_bits_y,
			delta_bits_z,
			num_anchors);
		
		ofstream outFileD(newname);
		
		if (! outFileD) {
			cerr << "Failed to open " << newname << " for writing!" << endl;
			return -8;
		}
		
		mesh.WriteToPly2Format(outFileD, 1); // write the delta-geometry
		
		ofstream anchorFile("anchors.hnd");
		mesh.WriteAnchorsToFile(anchorFile);
	}
	
	if (xyz_bits > 0) {
		char *newname = new char[strlen(argv[1]) + 100];
		char *noext = new char[strlen(argv[1]) + 1];
		if (*(strstr(argv[1], ".ply2"))) {
			strcpy(noext, argv[1]);
			noext[strlen(argv[1]) - 5] = '\0';
		}
		else {
			strcpy(noext, argv[1]);
		}
		
		sprintf(newname, "results/quantres_%s_%d.ply2", noext, xyz_bits);
		ofstream outFileQ(newname);
		if (! outFileQ) {
			cerr << "Failed to open " << newname << " for writing!" << endl;
			return -9;
		}
		
		mesh.ComputeXYZQuantizedGeometry(xyz_bits, xyz_bits, xyz_bits);
		mesh.WriteToPly2Format(outFileQ, 2); // write the xyz-quantized geometry
	}
	
	return 0;
}


int old_main(int argc, char *argv[]) {
	if (argc < 2)
		return -1;
	
	ifstream inFile(argv[1]);
	
	if (! inFile) {
		cerr << "Couldn't open file " << argv[1] << endl;
		return -3;
	}
	
	// reading the mesh
	TexturedMesh mesh;
	
	if (! mesh.ReadFromPly2Format(inFile)) {
		cerr << "Failed to read the mesh from supplied ply2 file" << endl;
		return -4;
	}
	
	cout << "before cleanup:" << endl;
	cout << "num vertices: " << mesh.GetNumberOfVertices() << endl;
	cout << "num triangles: " << mesh.GetNumberOfTriangles() << endl;
	
	cout << "no. of boundary vertices: " << mesh.GetNumberOfBoundaryVertices() << endl;
	
	// test 2-list
	//  mesh.PrintSecondOrderNeighborsList();
	
	// testing the matrix multiplication
	//  mesh.CreateAtAmatrix();
	
	//  ofstream lapFile("Lap.ijv");
	//  mesh.WriteLaplacian2IJV(lapFile);
	
	return 0;
}


/*
int old_main2(int argc, char *argv[]) {
switch (argc) {
case 1:
case 2:
case 3:
case 4:
case 5:
case 6:
cerr << "Not enough parameters" << endl;
printUsage();
return -1;
break;
case 7: // this is what we want: <progname> <filename> d_bits_x d_bits_y d_bits_z q_bits num_iterated anchors
break;
case 8:
default:
cerr << "Too many parameters!" << endl;
printUsage();
return -2;
break;
}

 // open the input file
 
  ifstream inFile(argv[1]);
  
   if (! inFile) {
   cerr << "Couldn't open file " << argv[1] << endl;
   return -3;
   }
   
	// reading the mesh
	TexturedMesh mesh;
	
	 if (! mesh.ReadFromPly2Format(inFile)) {
	 cerr << "Failed to read the mesh from supplied ply2 file" << endl;
	 return -4;
	 }
	 
	  cout << "before cleanup:" << endl;
	  cout << "num vertices: " << mesh.GetNumberOfVertices() << endl;
	  cout << "num triangles: " << mesh.GetNumberOfTriangles() << endl;
	  
	   cout << "no. of boundary vertices: " << mesh.GetNumberOfBoundaryVertices() << endl;
	   
		
		 //  mesh.CleanIsolatedVertices();
		 
		  //  cout << "after cleanup:" << endl;
		  //  cout << "num vertices: " << mesh.GetNumberOfVertices() << endl;
		  //  cout << "num triangles: " << mesh.GetNumberOfTriangles() << endl;
		  
		   
			mesh.Experiment1();
			
			 cout << "Executed Experiment1() function" << endl;
			 
			  int delta_bits_x = atoi(argv[2]);
			  int delta_bits_y = atoi(argv[3]);
			  int delta_bits_z = atoi(argv[4]);
			  int xyz_bits = atoi(argv[5]);
			  int num_anchors = atoi(argv[6]);
			  
			   //*************
			   ofstream commandFile("commandFile.csh");
			   commandFile << "matlab -nodesktop -nodisplay -nojvm <<EOF" << endl;
			   commandFile << "delta_bits_x=" << delta_bits_x << ";";
			   commandFile << "delta_bits_y=" << delta_bits_y << ";";
			   commandFile << "delta_bits_z=" << delta_bits_z << ";";
			   commandFile << "xyz_bits=" << xyz_bits << ";" << endl;
			   commandFile << "num_iterated_anchors=" << num_anchors << ";" << endl;
			   commandFile << "togscript;" << endl;
			   
				commandFile << "metrics_file = 'results/" << argv[1]<<"_"<<delta_bits_x<<"_"<<delta_bits_y<<"_"<<delta_bits_z<<"_"<<num_anchors<<".metrics';"<<endl;
				
				 commandFile << "metrics;" << endl;
				 commandFile << "exit" << endl;
				 commandFile << "EOF" << endl;
				 
				  commandFile.close();
				  
				   system("chmod u+x commandFile.csh");
				   system("commandFile.csh");
				   //*************
				   
					mesh.ReadExperiment();
					
					 cout << "Executed ReadExperiment() function" << endl;
					 
					  char *newname = new char[strlen(argv[1]) + 100];
					  char *noext = new char[strlen(argv[1]) + 1];
					  if (*(strstr(argv[1], ".ply2"))) {
					  strcpy(noext, argv[1]);
					  noext[strlen(argv[1]) - 5] = '\0';
					  }
					  else {
					  strcpy(noext, argv[1]);
					  }
					  
					   sprintf(newname, "results/deltares_%s_%d_%d_%d_%d.ply2", noext,delta_bits_x,delta_bits_y,delta_bits_z,num_anchors);
					   ofstream outFileD(newname);
					   
						if (! outFileD) {
						cerr << "Failed to open " << newname << " for writing!" << endl;
						return -8;
						}
						
						 mesh.WriteToPly2Format(outFileD, 1); // write the delta-geometry
						 
						  sprintf(newname, "results/quantres_%s_%d.ply", noext, xyz_bits);
						  ofstream outFileQ(newname);
						  if (! outFileQ) {
						  cerr << "Failed to open " << newname << " for writing!" << endl;
						  return -9;
						  }
						  
						   mesh.WriteToPly2Format(outFileQ, 2); // write the delta-geometry
						   
							
							 return 0;
							 }
*/


