#include "gtbmouse.h"
#ifndef WIN32
#include "point2.hpp"
#include "vector2.hpp"
#endif // WIN32


#ifdef OUTLINE
#define inline
#include <gtb/graphics/point2.ipp>
#undef inline
#endif


using namespace std;


BEGIN_GTBMOUSENS


const Point2 POINT2_ZERO(0.0, 0.0);


Point2::Point2()
{
	_p[0] = 0.0;
	_p[1] = 0.0;
}


Point2::Point2(const Point2 &p)
{
	_p[0] = p._p[0];
	_p[1] = p._p[1];
}


Point2::Point2(real_t px, real_t py)
{
	_p[0] = px;
	_p[1] = py;
}


Point2::Point2(const real_t p[2])
{
	_p[0] = p[0];
	_p[1] = p[1];
}


Point2 &Point2::scale(const Point2 &origin, real_t s)
{
	Vector2 t = origin - POINT2_ZERO;
	*this -= t;
	_p[0] *= s;
	_p[1] *= s;
	*this += t;
	return *this;
}


Point2 &Point2::rotate(real_t theta)
{
	real_t c = cos(theta);
	real_t s = sin(theta);
	real_t px = _p[0];
	real_t py = _p[1];
	_p[0] = (px * c) - (py * s);
	_p[1] = (px * s) + (py * c);
	return *this;
}


Point2 &Point2::rotate(const Point2 &origin, real_t theta)
{
	Vector2 t(origin._p);
	*this -= t;
	t.rotate(theta);
	*this += t;
	return *this;
}


Point2 Point2::centroid(const vector<Point2> &v)
{
	real_t cx = 0.0;
	real_t cy = 0.0;
	for (unsigned i = 0; i < v.size(); i++) {
		cx += v[i].x() / v.size();
		cy += v[i].y() / v.size();
	}
	return Point2(cx, cy);
}


END_GTBMOUSENS
