#include <GL/gl.h>
#include "vector2.hpp"

BEGIN_GTBMOUSENS


inline Point2 &Point2::operator=(const Point2 &p)
{
	_p[0] = p._p[0];
	_p[1] = p._p[1];
	return *this;
}


inline bool Point2::operator==(const Point2 &p) const
{
return false;
}


inline bool Point2::operator!=(const Point2 &p) const
{
	return !(*this == p);
}


inline bool Point2::is_zero() const
{
return false;
}


inline real_t Point2::x() const
{
	return _p[0];
}


inline real_t Point2::y() const
{
	return _p[1];
}


inline void Point2::set_x(real_t px)
{
	_p[0] = px;
}


inline void Point2::set_y(real_t py)
{
	_p[1] = py;
}


inline Point2 &Point2::reset(real_t px, real_t py)
{
	_p[0] = px;
	_p[1] = py;
	return *this;
}


inline real_t Point2::operator[](unsigned i) const
{
	assert(i < 2);
	return _p[i];
}


inline real_t &Point2::operator[](unsigned i)
{
	assert(i < 2);
	return _p[i];
}


inline Point2 &Point2::operator+=(const Vector2 &v)
{
	_p[0] += v._v[0];
	_p[1] += v._v[1];
	return *this;
}


inline Point2 &Point2::operator-=(const Vector2 &v)
{
	_p[0] -= v._v[0];
	_p[1] -= v._v[1];
	return *this;
}


inline Point2 operator+(const Point2 &p, const Vector2 &v)
{
	return Point2(p._p[0] + v._v[0],
		      p._p[1] + v._v[1]);
}


inline Vector2 operator-(const Point2 &p, const Point2 &q)
{
	return Vector2(p._p[0] - q._p[0],
		       p._p[1] - q._p[1]);
}


inline Point2 operator-(const Point2 &p, const Vector2 &v)
{
	return Point2(p._p[0] - v._v[0],
		      p._p[1] - v._v[1]);
}


inline Point2 &Point2::translate(const Vector2 &t)
{
	return *this += t;
}


inline Point2 &Point2::translate(real_t dx, real_t dy)
{
	_p[0] += dx;
	_p[1] += dy;
	return *this;
}


inline real_t Point2::distance(const Point2 &p, const Point2 &q)
{
	return (p - q).length();
}


inline real_t Point2::squared_distance(const Point2 &p, const Point2 &q)
{
	return (p - q).squared_length();
}


inline Point2 Point2::midpoint(const Point2 &A, const Point2 &B)
{
	return Point2((A.x() + B.x()) / 2.0,
		      (A.y() + B.y()) / 2.0);
}


inline void Point2::load() const
{
#ifdef REAL_IS_FLOAT
	glVertex2f(_p[0], _p[1]);
#else
	glVertex2d(_p[0], _p[1]);
#endif
}


inline std::istream &operator>>(std::istream &s, Point2 &p)
{
        return s >> p[0] >> p[1];
}


inline std::ostream &operator<<(std::ostream &s, const Point2 &p)
{
	return s << p[0] << ' ' << p[1];
}


END_GTBMOUSENS
