#ifndef GTB_RAY3_INCLUDED
#define GTB_RAY3_INCLUDED

#include "gtbmousecommon.h"
#include "point3.hpp"
#include "vector3.hpp"
#include <list>


BEGIN_GTBMOUSENS


//class Line3;
//class Polygon3;
class Box3;


class Ray3 {
public:
	Ray3();
	Ray3(const Ray3 &r);
	Ray3(const Point3 &p, const Vector3 &d);
	Ray3(const Point3 &p, const Point3 &q);
	Ray3 &operator=(const Ray3 &r);

	bool operator==(const Ray3 &r) const;
	bool operator!=(const Ray3 &r) const;

	Ray3 operator-() const;

	const Point3 &source() const;
        const Vector3 &direction() const;
	bool is_degenerate() const;

	const Point3 point(real_t t) const;

	// Parametric value of closest point on ray.
	real_t t(const Point3 &p) const;

//	Line3 supporting_line() const;
	bool contains(const Point3 &p) const;
	Point3 projection(const Point3 &p) const;

//	bool intersects(const Polygon3 &poly) const;

//	bool intersects(const Polygon3 &poly,
//			real_t &hit_time) const;

//	bool intersects(const Polygon3 &poly,
//			real_t &hit_time,
//			Point3 &hit_point) const;

	bool intersects(const Box3 &box) const;

	bool intersects(const Box3 &box, real_t &t1, real_t &t2) const;

protected:
	Point3 _source;
	Vector3 _direction;
};


extern const Ray3 RAY3_POSITIVE_X;
extern const Ray3 RAY3_NEGATIVE_X;
extern const Ray3 RAY3_POSITIVE_Y;
extern const Ray3 RAY3_NEGATIVE_Y;
extern const Ray3 RAY3_POSITIVE_Z;
extern const Ray3 RAY3_NEGATIVE_Z;


END_GTBMOUSENS

#ifndef OUTLINE
#include "ray3.ipp"
#endif

#endif // GTB_RAY3_INCLUDED
