#include "gtbmouse.h"
#ifndef WIN32
#include <gtb/graphics/vector3.hpp>
#include <gtb/graphics/point3.hpp>
#include <gtb/math/math.hpp>
#endif // WIN32


#ifdef OUTLINE
#define inline
#include <gtb/graphics/vector3.ipp>
#undef inline
#endif


BEGIN_GTBMOUSENS


const Vector3 VECTOR3_ZERO(0.0, 0.0, 0.0);
const Vector3 VECTOR3_POSITIVE_X(1.0, 0.0, 0.0);
const Vector3 VECTOR3_NEGATIVE_X(-1.0, 0.0, 0.0);
const Vector3 VECTOR3_POSITIVE_Y(0.0, 1.0, 0.0);
const Vector3 VECTOR3_NEGATIVE_Y(0.0, -1.0, 0.0);
const Vector3 VECTOR3_POSITIVE_Z(0.0, 0.0, 1.0);
const Vector3 VECTOR3_NEGATIVE_Z(0.0, 0.0, -1.0);


Vector3 &Vector3::rotate(const Vector3 &axis, real_t theta)
{
	// From Goldstein
	*this =	*this * cos(theta) +
		axis * this->dot(axis) * (1.0 - cos(theta)) -
		this->cross(axis) * sin(theta);
	return *this;
}


// Finds the rotation that takes V to W (Graphics Gems I, p. 531).
// (Here we pre-multiply, though, i.e., M * p, so we have to use the
// transpose of what's shown there.)
Matrix4 Vector3::rotation(const Vector3 &V, const Vector3 &W)
{
	Vector3 N = V.cross(W);
	N.normalize();

	Vector3 M = N.cross(V);
	M.normalize();

	Matrix4 Q(V.x(), V.y(), V.z(), 0.0,
		  M.x(), M.y(), M.z(), 0.0,
		  N.x(), N.y(), N.z(), 0.0,
		  0.0, 0.0, 0.0, 1.0);

	Vector3 W2 = Q * W;

	Matrix4 RT(W2.x(), -W2.y(), 0.0, 0.0,
		   W2.y(), W2.x(), 0.0, 0.0,
		   W2.z(), W2.z(), 1.0, 0.0,
		   0.0, 0.0, 0.0, 1.0);

	return Q.transpose() * RT * Q;
}


END_GTBMOUSENS
