//#include <gtb/error/error.hpp>
#include <GL/gl.h>
#include <ctype.h>


BEGIN_GTBMOUSENS


inline Vector3::Vector3()
{
#if 0
	_v[0] = 0.0;
	_v[1] = 0.0;
	_v[2] = 0.0;
#endif
}


inline Vector3::Vector3(const Vector3 &v)
{
	_v[0] = v._v[0];
	_v[1] = v._v[1];
	_v[2] = v._v[2];
}


inline Vector3::Vector3(real_t vx, real_t vy, real_t vz)
{
	_v[0] = vx;
	_v[1] = vy;
	_v[2] = vz;
}


inline Vector3::Vector3(const real_t v[3])
{
	_v[0] = v[0];
	_v[1] = v[1];
	_v[2] = v[2];
}


inline Vector3::Vector3(const real_t v) 
{
	_v[0] = v;
	_v[1] = v;
	_v[2] = v;
}

inline Vector3::Vector3(const Point3 &p)
{
	_v[0] = p._p[0];
	_v[1] = p._p[1];
	_v[2] = p._p[2];
}

inline Vector3 &Vector3::operator=(const Vector3 &v)
{
	_v[0] = v._v[0];
	_v[1] = v._v[1];
	_v[2] = v._v[2];
	return *this;
}


inline real_t Vector3::x() const
{
	return _v[0];
}


inline real_t Vector3::y() const
{
	return _v[1];
}


inline real_t Vector3::z() const
{
	return _v[2];
}


inline void Vector3::set_x(real_t vx)
{
	_v[0] = vx;
}


inline void Vector3::set_y(real_t vy)
{
	_v[1] = vy;
}


inline void Vector3::set_z(real_t vz)
{
	_v[2] = vz;
}


inline Vector3 &Vector3::reset(real_t vx, real_t vy, real_t vz)
{
	_v[0] = vx;
	_v[1] = vy;
	_v[2] = vz;
	return *this;
}


inline real_t Vector3::operator[](unsigned i) const
{
	assert(i < 3);
	return _v[i];
}


inline real_t &Vector3::operator[](unsigned i)
{
	assert(i < 3);
	return _v[i];
}


inline bool Vector3::operator==(const Vector3 &v) const
{
#if 0
	return (real::is_equal(_v[0], v._v[0]) &&
		real::is_equal(_v[1], v._v[1]) &&
		real::is_equal(_v[2], v._v[2]));
#else
    return false; // NOT implemented!
#endif
}


inline bool Vector3::operator!=(const Vector3 &v) const
{
	return !(*this == v);
}


inline bool Vector3::is_zero() const
{
#if 0
	return (real::is_zero(_v[0]) &&
		real::is_zero(_v[1]) &&
		real::is_zero(_v[2]));
#else
    return false; // NOT implemented!
#endif
}


inline Vector3 Vector3::operator-() const
{
	return Vector3(-_v[0],
		       -_v[1],
		       -_v[2]);
}


inline Vector3 &Vector3::operator+=(const Vector3 &v)
{
	_v[0] += v._v[0];
	_v[1] += v._v[1];
	_v[2] += v._v[2];
	return *this;
}


inline Vector3 &Vector3::operator-=(const Vector3 &v)
{
	_v[0] -= v._v[0];
	_v[1] -= v._v[1];
	_v[2] -= v._v[2];
	return *this;
}


inline Vector3 &Vector3::operator*=(real_t a)
{
	_v[0] *= a;
	_v[1] *= a;
	_v[2] *= a;
	return *this;
}


inline Vector3 &Vector3::operator*=(const Matrix4 &m)
{
	// Do we need vw?
	real_t vx, vy, vz, vw;
	vx = x() * m[0][0] + y() * m[0][1] + z() * m[0][2];
	vy = x() * m[1][0] + y() * m[1][1] + z() * m[1][2];
	vz = x() * m[2][0] + y() * m[2][1] + z() * m[2][2];
	vw = x() * m[3][0] + y() * m[3][1] + z() * m[3][2] + m[3][3];
//	assert(!real::is_zero(vw));
	return reset(vx / vw, vy / vw, vz / vw);
}


inline Vector3 &Vector3::operator/=(real_t a)
{
#if 0
	if (real::is_zero(a)) {
		//GTB_WARNING("division by 0 in Vector3::operator/=");
	}
#endif
	_v[0] /= a;
	_v[1] /= a;
	_v[2] /= a;
	return *this;
}


inline real_t Vector3::dot(const Vector3 &v) const
{
	return (_v[0] * v._v[0] +
		_v[1] * v._v[1] +
		_v[2] * v._v[2]);
}


inline real_t Vector3::dot(const Point3 &p) const
{
	return (_v[0] * p._p[0] +
		_v[1] * p._p[1] +
		_v[2] * p._p[2]);
}


inline real_t Vector3::operator*(const Vector3 &v) const
{
	return dot(v);
}


inline real_t Vector3::operator*(const Point3 &p) const
{
	return dot(p);
}


inline Vector3 operator+(const Vector3 &u, const Vector3 &v)
{
	return Vector3(u._v[0] + v._v[0],
		       u._v[1] + v._v[1],
		       u._v[2] + v._v[2]);
}


inline Vector3 operator-(const Vector3 &u, const Vector3 &v)
{
	return Vector3(u._v[0] - v._v[0],
		       u._v[1] - v._v[1],
		       u._v[2] - v._v[2]);
}


inline Vector3 operator*(real_t a, const Vector3 &v)
{
	return Vector3(a * v._v[0],
		       a * v._v[1],
		       a * v._v[2]);
}


inline Vector3 operator*(const Vector3 &v, real_t a)
{
	return a * v;
}


inline Vector3 operator*(const Matrix4 &m, const Vector3 &v)
{
	// Do we need vw?
	real_t vx, vy, vz, vw;
	vx = v.x() * m[0][0] + v.y() * m[0][1] + v.z() * m[0][2];
	vy = v.x() * m[1][0] + v.y() * m[1][1] + v.z() * m[1][2];
	vz = v.x() * m[2][0] + v.y() * m[2][1] + v.z() * m[2][2];
	vw = v.x() * m[3][0] + v.y() * m[3][1] + v.z() * m[3][2] + m[3][3];
//	assert(!real::is_zero(vw));
	return Vector3(vx / vw, vy / vw, vz / vw);
}


inline Vector3 operator/(const Vector3 &v, real_t a)
{
//	assert(!real::is_zero(a));
	return Vector3(v._v[0] / a,
		       v._v[1] / a,
		       v._v[2] / a);
}


inline Vector3 Vector3::cross(const Vector3 &v) const
{
	return Vector3((_v[1] * v._v[2]) - (_v[2] * v._v[1]),
		       (_v[2] * v._v[0]) - (_v[0] * v._v[2]),
		       (_v[0] * v._v[1]) - (_v[1] * v._v[0]));
}


inline real_t Vector3::length() const
{
	return sqrt(this->dot(*this));
}


inline real_t Vector3::squared_length() const
{
	return this->dot(*this);
}


inline Vector3 &Vector3::normalize()
{
	*this /= length();
	return *this;
}

inline Vector3 Vector3::normalized() const
{
    Vector3 r = *this;
    r.normalize();
    return r;
}

inline bool Vector3::is_normalized() const
{
//	return real::is_equal(length(), 1.0);
}


inline Vector3 &Vector3::scale(real_t a)
{
	*this *= a;
	return *this;
}


inline void Vector3::load_as_normal() const
{
	glNormal3d(_v[0], _v[1], _v[2]);
}


inline Vector3 &Vector3::transform(const Matrix4 &m)
{
	return *this *= m;
}


inline Vector3 &Vector3::affine_transform(const Matrix4 &m)
{
	real_t vx, vy, vz;
	vx = x() * m[0][0] + y() * m[0][1] + z() * m[0][2];
	vy = x() * m[1][0] + y() * m[1][1] + z() * m[1][2];
	vz = x() * m[2][0] + y() * m[2][1] + z() * m[2][2];
	return reset(vx, vy, vz);
}

inline void Vector3::flip()
{
	_v[0] = -_v[0];
	_v[1] = -_v[1];
	_v[2] = -_v[2];
}

inline Vector3 Vector3::flipped() const
{
    Vector3 r(*this);
    r.flip();
    return r;
}

inline void swap(Vector3 &a, Vector3 &b)
{
	Vector3 tmp(a);
	a = b;
	b = tmp;
}


inline std::istream &operator>>(std::istream &s, Vector3 &v)
{
        return s >> v[0] >> v[1] >> v[2];
}


inline std::ostream &operator<<(std::ostream &s, const Vector3 &v)
{
	return s << v[0] << ' ' << v[1] << ' ' << v[2];
}

#if 0
inline void Vector3::read(FILE *fp)
{
#ifdef REAL_IS_FLOAT
	read_float(&_v[0], fp);
	read_float(&_v[1], fp);
	read_float(&_v[2], fp);
#else
	float vx, vy, vz;
	read_float(&vx, fp);
	read_float(&vy, fp);
	read_float(&vz, fp);
	reset(vx, vy, vz);
#endif
}


inline void Vector3::write(FILE *fp) const
{
	write_float(_v[0], fp);
	write_float(_v[1], fp);
	write_float(_v[2], fp);
}
#endif

END_GTBMOUSENS
