#include "x3dExporter.h"

x3dExporter * x3dExporter::s_instance = NULL;

x3dExporter::x3dExporter(ostream & dstStream) :
    m_stream(dstStream),
    m_indent(0), m_inc(4),
    m_lastEOL(true), m_lastInc(false)
{
    assert(s_instance==NULL);s_instance = this;
    init();
    declare();
}

x3dExporter::~x3dExporter()
{
    m_stream << flush;
}

void x3dExporter::init()
{
    m_indentStr[0] = '\0';
    for (int i = 1; i < MAX_INDENT; i++) m_indentStr[i] = ' ';
}

void x3dExporter::inc()
{
    m_indentStr[m_indent] = ' ';
    m_indent += m_inc;
    if (m_indent > MAX_INDENT) m_indent = MAX_INDENT;
    m_indentStr[m_indent] = '\0';
}

void x3dExporter::dec()
{
    m_indentStr[m_indent] = ' ';
    m_indent -= m_inc;
    if (m_indent < 0) m_indent = 0;
    m_indentStr[m_indent] = '\0';
}

// Declarations:
void x3dExporter::declare()
{
    m_stream << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
    m_lastEOL = true;
}

// Process Elements:
void x3dExporter::startElement(const myString * tag)
{
    m_stream << m_indentStr << "<" << tag->c_str() << ">\n";
    m_lastEOL = true;
    inc();
}

void x3dExporter::endElement(const myString * tag)
{
    dec();
    m_stream << m_indentStr << "</" << tag->c_str() << ">\n";
    m_lastEOL = true;
}

void x3dExporter::emptyElement(const myString * tag)
{
    m_stream << m_indentStr << "<" << tag->c_str() << "/>\n";
    m_lastEOL = true;
}

void x3dExporter::openElement(const myString * tag)
{
    m_stream << m_indentStr << "<" << tag->c_str();
    m_lastEOL = false;
    m_lastInc = false;
}

void x3dExporter::closeElement()
{
    m_stream << ">\n";
    m_lastEOL = true;
    if (!m_lastInc) {
        inc();
        m_lastInc = true;
    }
}

void x3dExporter::closeEndElement()
{
    m_stream << "/>\n";
    m_lastEOL = true;
    if (m_lastInc) {
        dec();
        m_lastInc = false;
    }
}

// Process Attributes:
void x3dExporter::attribute(const myString * name,
                            const myString * value,
                            bool newLine)
{
    if (newLine) {
        m_stream << "\n";
        m_lastEOL = true;
        if (!m_lastInc) {
            inc();
            m_lastInc = true;
        }
        m_stream << m_indentStr;
    } else {
        m_lastInc = false;
        m_stream << " ";
    }

    if (value->hasSpace())
        m_stream << name->c_str() << "=\'" << value->c_str() << "\'";
    else m_stream << name->c_str() << "=\"" << value->c_str() << "\"";
    m_lastEOL = false;
}
