#ifndef X3DEXPORTER_H
#define X3DEXPORTER_H

#include <assert.h>
#include <fstream.h>
#include "myString.h"

class x3dExporter { 
public:
    x3dExporter(ostream & dstStream);
    ~x3dExporter();

    static x3dExporter * instance(void)
    {assert(s_instance); return(s_instance);}

    void init();
    
    // Declarations:
    void declare();
    
    // Process Elements:
    void startElement(const myString * tag);
    void endElement(const myString * tag);
    void emptyElement(const myString * tag);
    void openElement(const myString * tag);
    void closeElement();
    void closeEndElement();
    
    // Process Attributes:
    void attribute(const myString * name, const myString * value,
                   bool newLine = false);

    
private:
    static x3dExporter * s_instance;
    ostream & m_stream;
    
    void inc();
    void dec();
    
    int m_indent;
    int m_inc;
    
    #define MAX_INDENT 80
    char m_indentStr[MAX_INDENT+1];

    bool m_lastEOL;
    bool m_lastInc;
};

inline void exStartElement(const myString * tag)
{
    x3dExporter::instance()->startElement(tag);
}

inline void exEndElement(const myString * tag)
{
    x3dExporter::instance()->endElement(tag);
}

inline void exEmptyElement(const myString * tag)
{
    x3dExporter::instance()->emptyElement(tag);
}

inline void exOpenElement(const myString * tag)
{
    x3dExporter::instance()->openElement(tag);
}

inline void exCloseElement()
{
    x3dExporter::instance()->closeElement();
}

inline void exCloseEndElement()
{
    x3dExporter::instance()->closeEndElement();
}

inline void exAttribute(const myString * name, const myString * value)
{
    x3dExporter::instance()->attribute(name,value);
}

inline void exAttribute(const myString * name, const myString * value,
                   bool newLine)
{
    x3dExporter::instance()->attribute(name, value, newLine);
}

#endif
